@@ -7,7 +7,7 @@ class MACPAccessType(AbbreviatedEnum):
     cleanout_mainline = ("Cleanout Mainline", "ACOM")
     cleanout_property = ("Cleanout Property", "ACOP")
     junction_box = ("Junction Box", "AJB")
-    meter = ("Meter - Structure built to house flow measurement equipment.", "AM")
+    meter = ("Meter", "AM")
     manhole = ("Manhole", "AMH")
     other_special_structure = ("Other Special Structure", "AOC")
     wastewater_access = ("Wastewater Access", "AWA")
