@@ -2,7 +2,7 @@ from ...utils.abbreviated_enum import AbbreviatedEnum
 
 
 class ManholeUse(AbbreviatedEnum):
-    combined = ("Combined – Both sanitary sewage and stormwater flowing by gravity.", "CB")
+    combined = ("Combined", "CB")
     dam = ("Dam", "DP")
     force_main = ("Force Main", "FM")
     levee_gravity = ("Levee Gravity", "LG")
