@@ -1,6 +1,6 @@
 from ...utils.abbreviated_enum import AbbreviatedEnum
 
 
-class FlowDirection(AbbreviatedEnum):
+class SurveyDirection(AbbreviatedEnum):
     downstream = ("Downstream", "D")
     upstream = ("Upstream", "U")
