@@ -2,7 +2,7 @@ from ...utils.abbreviated_enum import AbbreviatedEnum
 
 
 class WeatherOptions(AbbreviatedEnum):
-    dry = ("Dry - No precipitation during survey", "1")
+    dry = ("Dry", "1")
     heavy_rain = ("Heavy Rain", "2")
     light_rain = ("Light Rain", "3")
     snow = ("Snow", "4")
