@@ -4,15 +4,22 @@ from typing import Type, TypeVar
 T = TypeVar("T", bound="AbbreviatedEnum")
 
 
-class AbbreviatedEnum(Enum):
-    def __init__(self, label: str, abbr: str):
-        self.label = label
-        self._abbreviation = abbr
+class AbbreviatedEnum(str, Enum):
+    def __new__(cls, label: str, abbr: str):
+        obj = str.__new__(cls, abbr)
+        obj._value_ = abbr
+        obj.label = label
+        obj._abbreviation = abbr
+        return obj
 
     @property
     def abbreviation(self) -> str:
         return self._abbreviation
 
+    @property
+    def full_name(self):
+        return f"{self.label} ({self.abbreviation})"
+
     @classmethod
     def from_abbreviation(cls: Type[T], abbr: str) -> T:
         for member in cls:
@@ -22,10 +29,10 @@ class AbbreviatedEnum(Enum):
 
     @classmethod
     def abbreviations(cls) -> dict[str, str]:
-        return {member.value: member.abbreviation for member in cls}
+        return {member.abbreviation: member.full_name for member in cls}
 
     def __str__(self):
-        return self.label
+        return self.abbreviation
 
     def __repr__(self):
         return f"{self.__class__.__name__}.{self.name}({self.label} - {self.abbreviation})"
